/**
 * @file toggle.h
 *
 * This is the main Toggle class and data interface definition
 * for communicating with the toggle .cogs program.
 * It is split into 2 parts: 1) data structs, and 2) Toggle class.
 * The toggle class has all code in this file.
 *
 * Copyright (c) 2012 by John Steven Denson
 * All rights MIT licensed.
 * See license statement at the end of this file for specific terms.
 */
 
#ifndef __TOGGLE_H
#define __TOGGLE_H

#include <stdio.h>
#include "propeller.h"

/* Header propeller.h includes stdint.h */
extern uint32_t _load_start_toggle_fw_cog[];
extern uint32_t _load_stop_toggle_fw_cog[];

/* This is the main mailbox struct.
 * Make sure it's volatile so optimizer doesn't throw out operations.
 */
typedef volatile struct toggle_mailbox {
    uint32_t pins;  /* pins to toggle */
    uint32_t wait;  /* time to sleep between toggles */
    uint32_t count; /* loop count */
} Mailbox_t;

/*
 * This defines the structure which we'll pass to the C cog.
 * It contains a small stack for working area, and the
 * mailbox which we use to communicate.
 */
struct par_struct {
    /* Provide a small stack for the COG */
    unsigned int stack[16];
    /* For passing messages between main program and .cogc code */
    Mailbox_t mailbox;
};

/*
 * Use this so that we can share the header with the .cogc code
 */
#if defined(__cplusplus)

class Toggle
{
public: /* Expose functions to the user. */
   
    /*
     * Constructor initializes the mailbox.
     */
    Toggle(uint32_t pins)
    {
        par.mailbox.wait = CLKFREQ;
        par.mailbox.count = 0;
        par.mailbox.pins = pins;
    }

    /*
     * Function to start up a new cog running the toggle code.
     * Code placed in the _load_start_toggle_fw_cog section.
     */
    int  start(void)
    {
    /* It's a little odd to use defines in a C++ program.
     * In this case we use the propeller-gcc compiler's definitions
     * so that we choose to do the right load automatically.
     */
    #if defined(__PROPELLER_XMMC__) || defined(__PROPELLER_XMM__)
        int size = _load_stop_toggle_fw_cog - _load_start_toggle_fw_cog;
        unsigned int cogbuffer[size];
        /* memcpy does bytes, so we copy numbytes * 4 = num ints. */      
        memcpy(cogbuffer, _load_start_toggle_fw_cog, size<<2);
    #else
        int *cogbuffer = (int*)_load_start_toggle_fw_cog;
    #endif
    
        /* When we start a cog we don't pass the stack, we pass the mailbox.
         * The stack may be used by the .cogc function.
         */
        return cognew(cogbuffer, &par.mailbox);
    }

    /* this is the update loop that changes the cogc wait time */
    void update(void)
    {
        int wait = par.mailbox.wait;
        wait = wait >> 1;
        if (wait < Toggle::MIN_GAP) {
            wait = CLKFREQ;
        }
        par.mailbox.wait = wait;
    }
    
    /* wait set accessor */
    void setWait(int wait)
    {
        par.mailbox.wait = wait;
    }
    
    /* wait get accessor */
    int getWait(void)
    {
        return par.mailbox.wait;
    }

    /* count get accessor */
    int getCount(void)
    {
        return par.mailbox.count;
    }


private: /* hide our data */

    /* This struct contains the stack and the mailbox.
     * The stack is typically not needed for single
     * function programs with few local variables.
     */
    struct par_struct par;
    
    /* Use enum instead of define so others can use the name.
     */
    enum { MIN_GAP = 400000 };
    
};

#endif
// __cplusplus

#endif
// __TOGGLE_H

/* +--------------------------------------------------------------------
 * | TERMS OF USE: MIT License
 * +--------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * +--------------------------------------------------------------------
 */
